/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Literals.java,v 0a9304b9cfca 2010/06/24 22:25:35 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.semantic.io.json.impl;

import javax.annotation.Nonnull;
import java.util.Date;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.net.URI;
import java.net.URISyntaxException;
import org.json.me.JSONException;
import org.json.me.JSONObject;
import org.json.me.JSONWriter;
import it.tidalwave.util.Id;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public final class Literals
  {
    private static final String KEY_DATATYPE = "datatype";
    private static final String KEY_TYPE = "type";
    private static final String KEY_VALUE = "value";
    
    private static final String TYPE_LITERAL = "literal";
    private static final String TYPE_URI = "uri";
    private static final String TYPE_BNODE = "bnode";

    private static final String DATE_PATTERN = "yyyy-MM-dd'T'hh:mm:ss";
    private static final String DATATYPE_DATETIME = "http://www.w3.org/2001/XMLSchema#dateTime";

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private Literals()
      {
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public static void marshalLiteral (final @Nonnull JSONWriter jw, final @Nonnull Object object)
      throws JSONException
      {
        if (object instanceof String)
          {
            marshalString(jw, (String)object);
          }
        else if (object instanceof Id)
          {
            try 
              {
                marshalUri(jw, new URI(((Id) object).stringValue()));
                //            marshalString(jw, ((Id)object).stringValue());
              }
            catch (URISyntaxException e)
              {
                throw new JSONException(e);
              }
          }
        else if (object instanceof Date)
          {
            marshalDate(jw, (Date)object);
          }
        else if (object instanceof URI)
          {
            marshalUri(jw, (URI)object);
          }
        else
          {
            jw.value("UNKNOWN " + object.getClass());
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public static void marshalReference (final @Nonnull JSONWriter jw, final @Nonnull Id id)
      throws JSONException
      {
        jw.object().key(KEY_VALUE).value(id.stringValue())
                   .key(KEY_TYPE).value(TYPE_BNODE)
                   .endObject();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public static Object unmarshalLiteral (final @Nonnull JSONObject object)
      throws JSONException
      {
        final String type = object.getString(KEY_TYPE);

        if (type.equals(TYPE_BNODE))
          {
            return new Reference(new Id(object.getString(KEY_VALUE)));
          }
        
        final String dataType = object.optString(KEY_DATATYPE);

        if (dataType.equals(Literals.DATATYPE_DATETIME))
          {
            try
              {
                final DateFormat df = new SimpleDateFormat(DATE_PATTERN);
                return df.parse(object.getString(KEY_VALUE));
              }
            catch (ParseException e)
              {
                throw new JSONException(e);
              }
          }

        if (dataType.equals(""))
          {
            return object.getString(KEY_VALUE);
          }

        throw new IllegalArgumentException("datatype=" + dataType);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private static void marshalString (final @Nonnull JSONWriter jw, final @Nonnull String string)
      throws JSONException
      {
        jw.object().key(KEY_VALUE).value(string)
                   .key(KEY_TYPE).value(TYPE_LITERAL)
                   .endObject();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private static void marshalDate (final @Nonnull JSONWriter jw, final @Nonnull Date date)
      throws JSONException
      {
        final DateFormat df = new SimpleDateFormat(DATE_PATTERN);
        jw.object().key(KEY_VALUE).value(df.format(date))
                   .key(KEY_TYPE).value(TYPE_LITERAL)
                   .key(KEY_DATATYPE).value(DATATYPE_DATETIME)
                   .endObject();// FIXME: needs datatype definition
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private static void marshalUri (final @Nonnull JSONWriter jw, final @Nonnull URI uri)
      throws JSONException
      {
        jw.object().key(KEY_VALUE).value(uri.toString())
                   .key(KEY_TYPE).value(TYPE_URI)
                   .endObject();
      }
  }
